
// GOBOYEXE.CPP

#include "goboy.h"
#include "fb.h"
#include "input.h"
#include "gb_sound.h"

#include <aknglobalnote.h>
#include <ezlib.h>


_LIT(KNoMemoryNote,"Out of memory. Please close any running applications, or restart phone.");

enum TServerUiList
	{
	EOptionsMenu,
	ELoadGameMenu,
	EDeleteGameMenu,
	ESelectKeyQuery
	};

enum TOptionsListItem
	{
	EOptionSave,
	EOptionLoad,
	EOptionDelete,
	EOptionSound,
	EOptionKeys,
	EOptionSwap,
	EOptionExit
	};



extern "C"
	{
void gb_main (int argc, char** argv);
	};


extern "C" void PostEvent(int aScancode, int aDown);


extern "C"
	{
	unsigned char* vidptr();
	};

extern "C"
	{
	unsigned char* vid_toggle_enable();
	};

extern "C"
	{
	void doSaveState(char*);
	};

extern "C"
	{
	void doLoadState(char*);
	};

extern "C" 
	{
	void set_paused(int aPaused);
	};

extern "C"
	{
	void exit_emulator();
	};

extern "C"
	{
	void gbsetSoundMode(int aMode);
	};

extern "C"
	{
	void gb_reset();
	};

extern "C"
	{
	void vid_close();
	};

RFs TheFs;
CGoBoy* goBoy;
CServerUi* iServerUi = NULL;
CGBSound* iGBSound = NULL;
char loadbuf[4096];

const TInt KUnzipBufferSize = 8192;

#define MAX_FRAMESKIP 4

int frameskip;

CGoBoy::CGoBoy()
:iScreenUtil(NULL)
	{
	frameskip = 0;
	}

CGoBoy::~CGoBoy()
	{
	gb_reset();
	vid_close();
	delete iGBSound;
	delete iFrame;
	delete iServerUi;
	delete iMenuCba;
	delete iCba;
	delete iScreen;
	delete iWindowGc;
	delete iWsScreen;
	iWsSession.Close();
	}

void CGoBoy::ConstructL(const TDesC& aCommandLine)
	{
	TInt nameLength = aCommandLine.Locate('*');
	if (nameLength == KErrNotFound)
		{
		iImageName = aCommandLine;
		}
	else
		{
		iImageName.Copy(aCommandLine.Left(nameLength-1));
		}
	iArgc = 0;
	iSwapButtons = EFalse;

	User::LeaveIfError(iWsSession.Connect());
    iWsScreen=new(ELeave) CWsScreenDevice(iWsSession);
    User::LeaveIfError(iWsScreen->Construct());
    User::LeaveIfError(iWsScreen->CreateContext(iWindowGc));
	iWsWindowGroup=RWindowGroup(iWsSession);
    User::LeaveIfError(iWsWindowGroup.Construct((TUint32)this));
    iWsWindowGroup.SetOrdinalPosition(0);
    iWsWindow=RWindow(iWsSession);
    User::LeaveIfError(iWsWindow.Construct(iWsWindowGroup, (TUint32)this));
    iWsWindow.Activate();


	iWsWindow.SetExtent(TPoint(0,0), TSize(176,208));

    iWsWindow.SetVisible(ETrue);
    StartWServEvents();

	iGBSound = new(ELeave)CGBSound();

	iGBSound->ConstructL();

	}

void CGoBoy::StartWServEvents()
    {
     iWsEventStatus = KRequestPending;
     iWsSession.EventReady(&iWsEventStatus);            // get Window Server events
     iRedrawEventStatus = KRequestPending;
     iWsSession.RedrawReady(&iRedrawEventStatus);      // get server-initiated redraw events
    }


void CGoBoy::CreateScreenL()
	{
	iScreen = new(ELeave)CFbsBitmap();
	TSize size = TSize(160,144);

	iScreen->Create(size, EColor4K);

	iMenuCba = new(ELeave)CFbsBitmap();
	iMenuCba->Create(TSize(176, 20), EColor4K);

	iServerUi = new(ELeave)CServerUi();
	iServerUi->ConstructL(iWsScreen, iScreen, iMenuCba, this);

	RProcess process;
	TFileName appName = process.FileName();
	TParsePtrC parse(appName);
	TFileName name(parse.DriveAndPath());
	name.Append(_L("goboyapp.mbm"));


	iCba = new(ELeave)CFbsBitmap();
	TInt err = iCba->Load(name, 1);
	if (err != KErrNone)
		{
		delete iCba;
		iCba = NULL;
		}

	iFrame = new(ELeave)CFbsBitmap();
	err = iFrame->Load(name, 3);
	if (err != KErrNone)
		{
		delete iFrame;
		iFrame = NULL;
		}



	PrepareScreen();
	SetDefaultKeys();
	LoadSettingsL();
	SetSoundMode();
	}


void CGoBoy::PrepareScreen()
	{
	CFbsBitmapDevice* bitDev = CFbsBitmapDevice::NewL(iScreen);
	CleanupStack::PushL(bitDev);
	CFbsBitGc* bitGc = CFbsBitGc::NewL();
	CleanupStack::PushL(bitGc);
	bitGc->Activate(bitDev);
	bitGc->SetBrushColor(TRgb(0,0,0));
	bitGc->SetBrushStyle(CGraphicsContext::ESolidBrush);
	bitGc->Clear();

	// Draw to screen
    iWindowGc->Activate(iWsWindow);
    TRect rect = TRect(iWsWindow.Size());
    iWsWindow.Invalidate(rect);
    iWsWindow.BeginRedraw(rect);

	iWindowGc->SetBrushColor(TRgb(0,0,0));
	iWindowGc->SetBrushStyle(CGraphicsContext::ESolidBrush);
	iWindowGc->Clear();
	iWindowGc->BitBlt(TPoint(0,188), iCba);
	if (iFrame)
		{
		iWindowGc->BitBlt(TPoint(0,0), iFrame);
		}

    iWsWindow.EndRedraw();
    iWindowGc->Deactivate();
    iWsSession.Flush();

	CleanupStack::PopAndDestroy(2);
	iScreenUtil = TBitmapUtil(iScreen);
	}


void CGoBoy::Go()
	{
	gb_main(iArgc, iArgv);
	}


void CGoBoy::UpdateScreen()
	{
	iScreenUtil.Begin(TPoint(0,0));
	TUint32* dest = (TUint32*)(iScreen->DataAddress());

	TUint32* src = (TUint32*)iVidPtr;

	TInt size = 144*80;

	while (size)
		{
		*dest = *src;
		src++;
		dest++;
		size--;
		}

	iScreenUtil.End();


	if (iServerUi->iIsActive)
		{
		iServerUi->Draw(iScreen);
		}

    iWindowGc->Activate(iWsWindow);
    TRect rect = TRect(iWsWindow.Size());
    iWsWindow.Invalidate(rect);
    iWsWindow.BeginRedraw(rect);

	iWindowGc->BitBlt(TPoint(8,16),iScreen);

    iWsWindow.EndRedraw();
    iWindowGc->Deactivate();
    iWsSession.Flush();

	}


void CGoBoy::GetEvents()
	{
     if (iWsEventStatus != KRequestPending)
      {
          iWsSession.GetEvent(iWsEvent);

		  if (iWsEvent.Type() == EEventKeyDown)
			  {
			  if (iServerUi->iIsActive)
				  {
				  iServerUi->KeyEvent(iWsEvent.Key()->iScanCode);
				  }
			  else
				  {
				  SendKeyDown(iWsEvent.Key()->iScanCode);
				  }
			  }
		  else if (iWsEvent.Type() == EEventKeyUp)
			  {
			  SendKeyUp(iWsEvent.Key()->iScanCode);
			  }
		  else if (iWsEvent.Type() == EEventFocusLost)
			  {
			  set_paused(1);
			  RThread thread;
			  thread.SetPriority(EPriorityMuchLess);
			  }
		  else if (iWsEvent.Type() == EEventFocusGained)
			  {
			  RThread thread;
			  thread.SetPriority(EPriorityNormal);
			  PrepareScreen();
			  }
          iWsEventStatus = KRequestPending;
          iWsSession.EventReady(&iWsEventStatus);
      }
	}




TInt CGoBoy::MapScanCode(TInt aScanCode)
	{
	if (aScanCode == iKeySelect[0])
		{
		return K_UP;
		}
	if (aScanCode == iKeySelect[1])
		{
		return K_DOWN;
		}
	if (aScanCode == iKeySelect[2])
		{
		return K_LEFT;
		}
	if (aScanCode == iKeySelect[3])
		{
		return K_RIGHT;
		}
	if (aScanCode == iKeySelect[4])
		{
		if (iSwapButtons)
			{
			return 's';	// Button B
			}
		else
			{
			return 'd';	// Button A
			}
		}
	if (aScanCode == iKeySelect[5])
		{
		if (iSwapButtons)
			{
			return 'd';	// Button A
			}
		else
			{
			return 's';	// Button B
			}
		}

	switch (aScanCode)
		{
		case EStdKeyEnter:
		case EStdKeyDevice1:
			return K_ENTER;	// Start
		case EStdKeySpace:
		case EStdKeyDevice0:
			return K_SPACE;	// Select
		case EStdKeyBackspace:
			return K_CTRL;
	default:
		return 0;
		}
	}


void CGoBoy::ProcessSaveL()
	{
	// GetNameForSaving
	TInt index = 1;
	TBool isUniqueName = EFalse;
	TFileName name;
	goBoy->AppendSaveGameDirectory(name);
	goBoy->AppendNameOfGame(name);


	TInt len = name.Length();
	while (!isUniqueName)
		{
		name.AppendFormat(_L(".%03d"), index);
		TEntry entry;
		if (TheFs.Entry(name, entry) == KErrNone)
			{
			index++;
			name.SetLength(len);
			}
		else
			{
			isUniqueName = ETrue;
			}
		}

	char fName[256];
	len = name.Length();
	for (TInt ii=0; ii<len; ii++)
		{
		fName[ii] = name[ii];
		}
	fName[ii] = 0;
	doSaveState(fName);

	TBuf<20> buffer;
	buffer.Format(_L("Game %03d saved"), index);
	ShowNoteL(_L("Saved..."), buffer);
	}

void CGoBoy::ProcessLoadL(TBool aDelete)
	{
	}



void CGoBoy::SendKeyDown(TInt aScanCode)
	{
	PostEvent(MapScanCode(aScanCode), 1);
	}

void CGoBoy::SendKeyUp(TInt aScanCode)
	{
	PostEvent(MapScanCode(aScanCode), 0);
	}

void CGoBoy::SetVidPtr(unsigned char* aVidPtr)
	{
	iVidPtr = aVidPtr;
	}

void CGoBoy::OpenFile(char* /*aName*/)
	{
	TInt loc;
	if ((loc = iImageName.Locate('#')) != KErrNotFound)
		{
		// Open Zip File
		TPtrC zipName = iImageName.Left(loc);
		TPtrC rest = iImageName.Mid(loc+1);
		loc = rest.Locate('#');
		TPtrC rest2 = rest.Mid(loc+1);
		loc = rest2.Locate('#');
		TLex lex(rest2.Left(loc));
		lex.Val(iZipOffset);
		TPtrC rest3 = rest2.Mid(loc+1);
		loc = rest3.Locate('#');
		TLex lex2(rest3.Left(loc));
		lex2.Val(iZipCSize);
		TLex lex3(rest3.Mid(loc+1));
		lex3.Val(iZipUSize);

		TInt err = iStream.Open(TheFs, zipName, EFileRead);
		}
	else
		{
		TInt err = iStream.Open(TheFs, iImageName, EFileRead);
		iZipOffset = 0;
		}
	}

void CGoBoy::CloseFile()
	{
	iStream.Close();
	}

void CGoBoy::ReadL(unsigned char* aDest, TInt aSize)
	{
	if (iZipOffset == 0)
		{
		iStream.ReadL(aDest, aSize);
		}
	else
		{
		// Unpack zip file
		HBufC8* buffer = HBufC8::NewMaxLC(KUnzipBufferSize);
		TPtr8 bufPtr = buffer->Des();

		TInt totalSize = iZipCSize;
		TInt next = KUnzipBufferSize;
		if (totalSize < next)
			{
			next = totalSize;
			}

		iStream.ReadL(iZipOffset);

		iStream.ReadL(bufPtr,next);

		z_stream stream;
		int err;

		stream.next_in = (Bytef*)bufPtr.Ptr();
		stream.avail_in = (uInt)next;

		stream.next_out = aDest;
		stream.avail_out = iZipUSize;

		stream.zalloc = (alloc_func)0;
		stream.zfree = (free_func)0;

		err = inflateInit2(&stream, -15);
		if (err != Z_OK)
			{
			User::Leave(KErrCorrupt);
			};

		while (err == Z_OK && totalSize > 0)
			{
			err = inflate(&stream, Z_SYNC_FLUSH);
			totalSize -= next;
			if (totalSize > 0)
				{
				next = KUnzipBufferSize;
				if (totalSize < next)
					{
					next = totalSize;
					}
				iStream.ReadL(bufPtr,next);
				stream.next_in = (Bytef*)bufPtr.Ptr();
				stream.avail_in = (uInt)next;
				}
			}

		err = inflateEnd(&stream);


		CleanupStack::PopAndDestroy(buffer);
		}
	}

int CGoBoy::FileSize(char* /*aName*/)
	{
	TInt loc;
	if ((loc = iImageName.Locate('#')) != KErrNotFound)
		{
		TInt size;
		TPtrC rest = iImageName.Mid(loc+1);
		loc = rest.Locate('#');
		TPtrC rest2 = rest.Mid(loc+1);
		loc = rest2.Locate('#');
		TPtrC rest3 = rest2.Mid(loc+1);
		loc = rest3.Locate('#');
		TLex lex3(rest3.Mid(loc+1));
		lex3.Val(size);
		return size;
		}
	else
		{
		TEntry entry;
		TInt err = TheFs.Entry(iImageName, entry);
		if (err == KErrNone)
			{
			return entry.iSize;
			}
		else
			{
			return 0;
			}
		}
	}

void CGoBoy::ShowNoteL(const TDesC& aTitle, const TDesC& aNote)
	{
	iServerUi->CreateNoteL(aTitle, aNote);
	iServerUi->Activate(ETrue);
	set_paused(1);
	}

void CGoBoy::AppendSaveGameDirectory(TDes& aName)
	{
	RProcess process;
	TFileName appName = process.FileName();
	TParsePtrC parse(appName);
	TFileName name(parse.DriveAndPath());
	name.Append(_L("save\\"));
	TheFs.MkDirAll(name);
	aName.Append(name);
	}

void CGoBoy::AppendNameOfGame(TDes& aName)
	{
	TInt loc;
	if ((loc = iImageName.Locate('#')) != KErrNotFound)
		{
		// Open Zip File
		TPtrC zipName = iImageName.Left(loc);
		TPtrC rest = iImageName.Mid(loc+1);
		loc = rest.Locate('#');
		TParsePtrC parse(rest.Left(loc));
		aName.Append(parse.Name());
		}
	else
		{
		TParsePtrC parse(iImageName);
		aName.Append(parse.Name());
		}
	}



extern "C" void memcpy()
	{
	}


extern "C" void unzip()
	{
	}

extern "C" void ev_poll()
	{
	// Update screen and check for events
	if (!vid_toggle_enable())
		{
		goBoy->UpdateScreen();
		}
	goBoy->GetEvents();
	}


extern "C" void ev_poll_paused()
	{
	goBoy->Redraw(ETrue);
	goBoy->GetEvents();
	}

extern "C" void begin_paused()
	{
	goBoy->OptionsMenuL();
	}


extern "C" void signal()
	{
	}

extern "C" void pcm()
	{
	}


extern "C" void sys_sleep()
	{
	}

extern "C" void sys_elapsed()
	{
	}

extern "C" void sys_timer()
	{
	}

extern "C" void sys_checkdir()
	{
	}

extern "C" void sys_initpath()
	{
	}

extern "C" void sys_sanitize()
	{
	}



extern "C" void pcm_exports()
	{
	}

extern "C" void pcm_close()
	{
	}


extern "C" void vid_preinit()
	{
	}



extern "C" void vid_begin()
	{
	}

extern "C" void vid_end()
	{
	}

extern "C" void vid_setpal(int i, int r, int g, int b)
	{
	}


extern "C" void vid_settitle(char *title)
	{
	}


extern "C" void vid_exports()
	{
	}


extern "C" void joy_exports()
	{
	}

extern "C" void gb_vidset(unsigned char* aPtr)
	{
	goBoy->SetVidPtr(aPtr);
	}


extern "C" void gb_openfile(char* aName)
	{
	goBoy->OpenFile(aName);
	}

extern "C" void gb_closefile()
	{
	goBoy->CloseFile();
	}

extern "C" void gb_readfile(unsigned char* aDest, int aSize)
	{
	goBoy->ReadL(aDest, aSize);
	}

extern "C" int gb_filesize(char* aName)
	{
	return goBoy->FileSize(aName);
	}

extern "C" void gb_note(char* text)
	{
	TBuf<128> buf;
	while ((*text) != 0)
		{
		buf.Append(*text);
		text++;
		}
	}

extern "C" void gb_notenum(char* text, TInt num)
	{
	TBuf<128> buf;
	while ((*text) != 0)
		{
		buf.Append(*text);
		text++;
		}
	buf.AppendFormat(_L(":%d"), num);
	}


extern "C" void* gb_malloc(int aSize)
	{
	void* ptr =  User::Alloc(aSize);
	if (ptr == 0)
		{
		CAknGlobalNote* note = CAknGlobalNote::NewLC();
		note->ShowNoteL(EAknGlobalInformationNote, KNoMemoryNote);
		CleanupStack::PopAndDestroy();
		User::Leave(KErrNoMemory);
		}
	return ptr;
	}

extern "C" void gb_free(void *aPtr)
	{
	delete aPtr;
	}


extern "C" char* gb_getbatteryfilename()
	{
	HBufC* nameBuf = HBufC::NewLC(256);
	TPtr name = nameBuf->Des();
	goBoy->AppendSaveGameDirectory(name);
	goBoy->AppendNameOfGame(name);

	name.Append(_L(".bt"));

	TInt len = name.Length();
	char* buffer = (char*)User::Alloc(len + 1);
	for (TInt ii=0; ii<len; ii++)
		{
		buffer[ii] = name[ii];
		}
	buffer[ii] = 0;
	CleanupStack::PopAndDestroy();	// nameBuf

	return (char*)buffer;
	}

extern "C" char* gb_buf()
	{
	return loadbuf;
	}

extern "C" void gb_setbuttons(int aSettings)
	{
	goBoy->iSwapButtons = aSettings;
	}

extern "C" int gb_getbuttons()
	{
	return goBoy->iSwapButtons;
	}
	
extern "C" void gb_submitsound(char* aBuffer, int aSize)
	{
	TBool terminated = iGBSound->SubmitAudio(aBuffer, aSize);
	if (terminated)
		{
		iServerUi->CreateNoteL(_L("Sound has stopped"), _L("Please register to"));
		iServerUi->AddListItem(_L("enjoy sound support"));
		iServerUi->Activate(ETrue);
		}
	}

extern "C" void gb_leave(int aLeave)
	{
	User::Leave(aLeave);
	}


LOCAL_C void StartL();

void CGoBoy::SetDefaultKeys()
	{
	iKeySelect[0] = EStdKeyUpArrow;
	iKeySelect[1] = EStdKeyDownArrow;
	iKeySelect[2] = EStdKeyLeftArrow;
	iKeySelect[3] = EStdKeyRightArrow;
	iKeySelect[4] = EStdKeyDevice3;
	iKeySelect[5] = '5';
	}



void CGoBoy::OptionsMenuL()
	{
	iServerUi->CreateList(EOptionsMenu, _L("Select option"));
	iServerUi->AddListItem(_L("Save"));
	iServerUi->AddListItem(_L("Load..."));
	iServerUi->AddListItem(_L("Delete..."));

	switch (iSoundMode)
		{
		case 0:
		iServerUi->AddListItem(_L("Sound : OFF")); break;
		case 1:
		iServerUi->AddListItem(_L("Sound : ON")); break;
		case 2:
		iServerUi->AddListItem(_L("Sound : LOUD")); break;
		}
	iServerUi->AddListItem(_L("Select Keys"));
	iServerUi->AddListItem(_L("Swap Buttons"));
	iServerUi->AddListItem(_L("Exit"));
	iServerUi->Activate(ETrue);
	}

void CGoBoy::LoadGameMenuL(TBool aDelete)
	{
	set_paused(1);
	if (aDelete)
		{
		iServerUi->CreateList(EDeleteGameMenu, _L("Delete Game..."));
		}
	else
		{
		iServerUi->CreateList(ELoadGameMenu, _L("Load Game..."));
		}
	TFileName name;
	goBoy->AppendSaveGameDirectory(name);
	TFileName game;
	goBoy->AppendNameOfGame(game);
	game.Append(_L(".???"));

	RDir dir;
	dir.Open(TheFs, name, 0);
	TEntry entry;
	while (dir.Read(entry) == KErrNone)
		{
		if (entry.iName.Match(game) == 0)
			{
			TBuf<80> listItem;
			listItem.Append(entry.iName.Mid(entry.iName.Length()-3));
			TBuf<40> fmt;
			_LIT(KFormatTxt," %H%:1%T %/0%4%/1%5%/3");
			entry.iModified.FormatL(fmt, KFormatTxt);
			listItem.Append(fmt);
			iServerUi->AddListItem(listItem);
			}
		}
	dir.Close();
	iServerUi->Activate(ETrue);
	}


void CGoBoy::ListItemSelection(TInt aListId, TInt aIndex)
	{
	if (aListId == EOptionsMenu)
		{
		set_paused(0);
		switch (aIndex)
			{
			case -1:
				break;
			case EOptionSound:
				iSoundMode++;
				if (iSoundMode == 2)
					{
					iSoundMode = 0;
					}
				SetSoundMode();
				SaveSettingsL();
				break;
			case EOptionSave:
				ProcessSaveL();
				break;
			case EOptionLoad:
				LoadGameMenuL();
				break;
			case EDeleteGameMenu:
				LoadGameMenuL(ETrue);
				break;
			case EOptionKeys:
				SelectKeysL(0);
				break;
			case EOptionSwap:
				iSwapButtons = !iSwapButtons;
				SaveSettingsL();
				break;
			case EOptionExit:
				exit_emulator();
				break;
			}
		}
	if (aListId == ELoadGameMenu)
		{
		set_paused(0);
		if (aIndex != -1)
			{
			LoadGameIndex(aIndex, EFalse);
			}
		}
	if (aListId == EDeleteGameMenu)
		{
		set_paused(0);
		if (aIndex != -1)
			{
			LoadGameIndex(aIndex, ETrue);
			}
		}
	}

void CGoBoy::LoadGameIndex(TInt aIndex, TBool aDelete)
	{
	TFileName name;
	goBoy->AppendSaveGameDirectory(name);


	TFileName game;
	goBoy->AppendNameOfGame(game);
	game.Append(_L(".???"));

	TFileName fullname;
	goBoy->AppendSaveGameDirectory(fullname);
	goBoy->AppendNameOfGame(fullname);
	fullname.Append('.');


	RDir dir;
	dir.Open(TheFs, name, 0);
	TEntry entry;
	while (dir.Read(entry) == KErrNone)
		{
		if (entry.iName.Match(game) == 0)
			{
			TBuf<80> listItem;
			listItem.Append(entry.iName.Mid(entry.iName.Length()-3));
			if (aIndex == 0)
				{
				fullname.Append(listItem);
				break;
				}
			aIndex--;
			}
		}
	dir.Close();




		if (aDelete)
			{
			TheFs.Delete(fullname);
			TBuf<80> buffer(_L("Game "));
			TParsePtrC parse(fullname);
			buffer.Append(parse.Ext().Mid(1));
			buffer.Append(_L(" deleted"));
			ShowNoteL(_L("Information"), buffer);
			}
		else
			{
			char fName[256];
			TInt len = fullname.Length();
			for (TInt ii=0; ii<len; ii++)
				{
				fName[ii] = fullname[ii];
				}
			fName[ii] = 0;

			doLoadState(fName);
			}


	}

void CGoBoy::SelectKeysL(TInt aKeyNo)
	{
	TBuf<16> keyName;
	switch (aKeyNo)
		{
		case 0: keyName.Copy(_L("Up")); break;
		case 1: keyName.Copy(_L("Down")); break;
		case 2: keyName.Copy(_L("Left")); break;
		case 3: keyName.Copy(_L("Right")); break;
		case 4: keyName.Copy(_L("Button A")); break;
		case 5: keyName.Copy(_L("Button B")); break;
		}
	iSelectKeyNo = aKeyNo;

	iServerUi->CreateKeySelectQueryL(ESelectKeyQuery, _L("Select Key for..."), keyName);
	iServerUi->Activate(ETrue);
	set_paused(1);
	}


void CGoBoy::KeySelect(TInt aId, TInt aKey)
	{
	if (aId == ESelectKeyQuery)
		{
		iKeySelect[iSelectKeyNo] = aKey;
		if (iSelectKeyNo < 5)
			{
			iSelectKeyNo++;
			SelectKeysL(iSelectKeyNo);
			}
		else
			{
			// Set Keys to emulator
			SaveSettingsL();
			set_paused(0);
			}
		}
	}


void CGoBoy::SetSoundMode()
	{
	switch (iSoundMode)
		{
		case 0:
			iGBSound->SetSoundOn(EFalse);
			break;
		case 1:
			iGBSound->SetSoundOn(ETrue);
			gbsetSoundMode(0);
			break;
		case 2:
			iGBSound->SetSoundOn(ETrue);
			gbsetSoundMode(1);
			break;
		}
	}



void CGoBoy::CbaPressed(TInt aListId, TCbaLabel)
	{
	if (aListId == -1)
		{
		set_paused(0);
		}
	}


void CGoBoy::Redraw(TBool aMenu)
	{
	if (aMenu)
		{

		iServerUi->Draw(iScreen);


	    iWindowGc->Activate(iWsWindow);
		TRect rect = TRect(iWsWindow.Size());
		iWsWindow.Invalidate(rect);
		iWsWindow.BeginRedraw(rect);

		iWindowGc->BitBlt(TPoint(8,16),iScreen);
		iWindowGc->BitBlt(TPoint(0,188), iMenuCba);


		iWsWindow.EndRedraw();
		iWindowGc->Deactivate();
		iWsSession.Flush();

		}
	else
		{
	    iWindowGc->Activate(iWsWindow);
		TRect rect = TRect(iWsWindow.Size());
		iWsWindow.Invalidate(rect);
		iWsWindow.BeginRedraw(rect);

		iWindowGc->BitBlt(TPoint(8,16),iScreen);
		iWindowGc->BitBlt(TPoint(0,188), iCba);


		iWsWindow.EndRedraw();
		iWindowGc->Deactivate();
		iWsSession.Flush();
		}
	}

void CGoBoy::SaveSettingsL()
	{
	TFileName name;
	AppendSaveGameDirectory(name);
	name.Append(_L("GoBoy.ini"));
	RFileWriteStream stream;
	TInt err = stream.Replace(TheFs, name, EFileWrite);
	if (err == KErrNone)
		{
		stream.WriteUint32L(iSoundMode);
		stream.WriteUint32L(iKeySelect[0]);
		stream.WriteUint32L(iKeySelect[1]);
		stream.WriteUint32L(iKeySelect[2]);
		stream.WriteUint32L(iKeySelect[3]);
		stream.WriteUint32L(iKeySelect[4]);
		stream.WriteUint32L(iKeySelect[5]);
		stream.WriteUint32L(iSwapButtons);

		stream.CommitL();
		stream.Close();
		}

	}

void CGoBoy::LoadSettingsL()
	{
	TFileName name;
	AppendSaveGameDirectory(name);
	name.Append(_L("GoBoy.ini"));
	RFileReadStream stream;
	TInt err = stream.Open(TheFs, name, EFileRead);
	if (err == KErrNone)
		{
		iSoundMode = stream.ReadUint32L();
		iKeySelect[0] = stream.ReadUint32L();
		iKeySelect[1] = stream.ReadUint32L();
		iKeySelect[2] = stream.ReadUint32L();
		iKeySelect[3] = stream.ReadUint32L();
		iKeySelect[4] = stream.ReadUint32L();
		iKeySelect[5] = stream.ReadUint32L();
		iSwapButtons = stream.ReadUint32L();
		stream.Close();
		}
	}


GLDEF_C TInt E32Main()
	{	
	__UHEAP_MARK; // mark heap state
    
	CTrapCleanup* TheTrapCleanup=CTrapCleanup::New();
	TRAPD(error, StartL());
    delete TheTrapCleanup;
	
	__UHEAP_MARKEND;
	
	return(error);
	}

#if defined(__WINS__)
GLDEF_C TInt E32Dll(TDllReason)
	{
	return(KErrNone);
	}
#endif


LOCAL_C void StartL()
	{
	// Start by renaming our thread
	RThread thread;
	thread.Rename(_L("GOBOYEXE"));

	RProcess process;
	TFileName commandLine;
	process.CommandLine(commandLine);

	TheFs.Connect();

	goBoy = new(ELeave)CGoBoy();
	goBoy->ConstructL(commandLine);
	goBoy->CreateScreenL();

	TRAPD(err, goBoy->Go());

	delete goBoy;
	TheFs.Close();
	}


